@@ -184,7 +184,7 @@ def array_encoder(orig, line='', level=0, separator_arrays=' ', compress_arrays=
 def encoder(orig, escaped_strings=True, dotBoolean=True, compress_arrays=True, max_array_chars=79):
     if isinstance(orig, (bool, np.bool_)):
         if dotBoolean:
-            tmp = '.' + str(orig) + '.'
+            tmp = '.' + str(orig).lower() + '.'
         else:
             tmp = str(orig)
     elif escaped_strings and isinstance(orig, str):
@@ -984,7 +984,10 @@ class NamelistFile(NamelistName):
                     ):
                         # handle split_arrays functionality by looping over individual segments
                         if not self.split_arrays:
-                            kid0, meKid0 = [kid], [meKid],
+                            kid0, meKid0 = (
+                                [kid],
+                                [meKid],
+                            )
                         elif callable(
                             self.split_arrays
                         ):  # this cannot be saved, but it is useful for development of the _split_arrays_function
@@ -1144,7 +1147,7 @@ class sparray(object):
         return tuple(_index)
 
     def _setitem_base(self, index, value):
-        """ set value to position given in index, where index is a tuple. """
+        """set value to position given in index, where index is a tuple."""
         # adjust number of dimensions
         index = tuple(map(int, index))
         if len(index) > len(self.shape):
@@ -1164,7 +1167,7 @@ class sparray(object):
         self.__data[index] = np.atleast_1d([value]).astype(self.dtype)
 
     def __setitem__(self, index, value):
-        """ set value to position given in index, where index is a tuple. """
+        """set value to position given in index, where index is a tuple."""
         index = self._index_offset(index, -1)
         if isinstance(value, np.ndarray):
             _index = list(index)
@@ -1175,7 +1178,7 @@ class sparray(object):
             self._setitem_base(index, value)
 
     def __getitem__(self, index):
-        """ get value at position given in index, where index is a tuple. """
+        """get value at position given in index, where index is a tuple."""
         index = self._index_offset(index, -1)
         try:
             return self.dense()[index]
@@ -1183,7 +1186,7 @@ class sparray(object):
             return index * 0 + self.default
 
     def __delitem__(self, index):
-        """ index is tuples of element to be deleted. """
+        """index is tuples of element to be deleted."""
         index = self._index_offset(index)
         if index in self.__data:
             del self.__data[index]
@@ -1230,7 +1233,7 @@ class sparray(object):
 
     def fortran(self, index, value):
         # Drop leading zeros otherwise the index is evaluated as an octal.
-        index = delete_leading_zeros(index)
+        index = _delete_leading_zeros(index)
         index = eval('(' + re.sub('([0-9]+):', r'\1', index.split('(')[-1].strip(')')) + ')')
         self.__setitem__(index, value)
         return self
@@ -1561,8 +1564,11 @@ class sparray(object):
 
     __hash__ = None
 
+    def copy(self):
+        return copy.deepcopy(self)
 
-def delete_leading_zeros(number_str):
+
+def _delete_leading_zeros(number_str):
     """
     Delete leading zeros from array indices
 
