@@ -23,7 +23,7 @@ import omas
 from omas import ODS
 from omfit_classes.omfit_base import OMFITtmp, OMFITmodule, OMFITtree
 from omfit_classes.omfit_json import OMFITjson, OMFITsettings, SettingsName
-from omfit_classes.omfit_omas_utils import setup_hardware_overlay_cx, add_hardware_to_ods
+from omfit_classes.omfit_omas_utils import setup_hardware_overlay_cx, add_hardware_to_ods, omas_eq1d_pretty_name
 from omfit_classes.omfit_formatter import omfit_formatter
 from omfit_classes.utils_fusion import is_device
 from omfit_classes.omfit_mds import translate_MDSserver
@@ -148,7 +148,9 @@ def add_command_box_tab():
 
 
 # Data loading and related helpers --------------------------------------------------------------------------------
-def parse_load_cx_ods_kw(gfile=None, device=None, shot=None, t=None, efitid=None, index=0, no_empty=None, settings=None):
+def parse_load_cx_ods_kw(
+    gfile=None, device=None, shot=None, t=None, efitid=None, index=0, no_empty=None, settings=None, minimal_eq_data=None
+):
     """
     Handles setting defaults for keywords to load_cx_ods()
 
@@ -195,11 +197,13 @@ def parse_load_cx_ods_kw(gfile=None, device=None, shot=None, t=None, efitid=None
         efitid = the_case.setdefault('efit_tree', None)
     if no_empty is None:
         no_empty = settings.setdefault('no_empty', True)
+    if minimal_eq_data is None:
+        minimal_eq_data = settings.setdefault('minimal_eq_data', True)
 
-    return gfile, device, shot, efitid, no_empty, ods_tag
+    return gfile, device, shot, efitid, no_empty, minimal_eq_data, ods_tag
 
 
-def load_cx_ods(gfile=None, device=None, shot=None, t=None, efitid=None, index=0, no_empty=None):
+def load_cx_ods(gfile=None, device=None, shot=None, t=None, efitid=None, index=0, no_empty=None, minimal_eq_data=None):
     """
     Puts an ODS with CX info in the tree
 
@@ -219,12 +223,27 @@ def load_cx_ods(gfile=None, device=None, shot=None, t=None, efitid=None, index=0
     :param index: int
         Index of the ODS, for managing multi-shot overlays
 
+    :param no_empty: bool
+        Filter out equilibrium time-slices that have 0 current or 0 boundary outline points.
+        (this is passed to multi_efit_to_omas() via setup_hardware_overlay_cx())
+
+    :minimal_eq_data: bool
+        If the source is MDSplus, load only the minimal dataset needed to power the equilibrium cross section plot.
+
     :return: ODS instance
     """
     printd(f'  loading cx ods for index {index}', topic='omfit_efitviewer')
     _, _, out, settings, _, _ = pick_output_location()
-    gfile, device, shot, efitid, no_empty, ods_tag = parse_load_cx_ods_kw(
-        gfile=gfile, device=device, shot=shot, t=t, efitid=efitid, index=index, no_empty=no_empty, settings=settings
+    gfile, device, shot, efitid, no_empty, minimal_eq_data, ods_tag = parse_load_cx_ods_kw(
+        gfile=gfile,
+        device=device,
+        shot=shot,
+        t=t,
+        efitid=efitid,
+        index=index,
+        no_empty=no_empty,
+        settings=settings,
+        minimal_eq_data=minimal_eq_data,
     )
 
     if isinstance(gfile, str):
@@ -236,15 +255,21 @@ def load_cx_ods(gfile=None, device=None, shot=None, t=None, efitid=None, index=0
         gfile_str = None
 
     if gfile is None:
-        load_cmd = "setup_hardware_overlay_cx({}, {}, {}, efitid={}, default_load=False, no_empty={})".format(
-            repr(device), repr(shot), repr(t), repr(efitid), no_empty
+        load_cmd = "setup_hardware_overlay_cx({}, {}, {}, efitid={}, default_load=False, no_empty={}, minimal_eq_data={})".format(
+            repr(device), repr(shot), repr(t), repr(efitid), no_empty, minimal_eq_data
+        )
+        ods = out[ods_tag] = setup_hardware_overlay_cx(
+            device, shot, t, efitid=efitid, default_load=False, quiet=True, no_empty=no_empty, minimal_eq_data=minimal_eq_data
         )
-        ods = out[ods_tag] = setup_hardware_overlay_cx(device, shot, t, efitid=efitid, default_load=False, quiet=True, no_empty=no_empty)
         ods['dataset_description.ids_properties.source'] = 'MDSplus'
     else:
         printd(f' Using a gfile or set with type {type(gfile)} to setup cx data.', topic='omfit_efitviewer')
-        ods = out[ods_tag] = setup_hardware_overlay_cx(device, geqdsk=gfile, default_load=False, quiet=True, no_empty=no_empty)
-        load_cmd = "setup_hardware_overlay_cx({}, geqdsk={}, default_load=False, no_empty={})".format(repr(device), gfile_str, no_empty)
+        ods = out[ods_tag] = setup_hardware_overlay_cx(
+            device, geqdsk=gfile, default_load=False, quiet=True, no_empty=no_empty, minimal_eq_data=minimal_eq_data
+        )
+        load_cmd = "setup_hardware_overlay_cx({}, geqdsk={}, default_load=False, no_empty={}, minimal_eq_data={})".format(
+            repr(device), gfile_str, no_empty, minimal_eq_data
+        )
         ods['dataset_description.ids_properties.source'] = gfile_str
 
     settings['cases'][index]['instructions_for_loading_ods'] = load_cmd
@@ -409,7 +434,7 @@ def plot_scaled_boundary_overlay(
     kw.pop('ods', None)  # We don't need this,but we have to take it to have a consistent call signature
 
     if ax is None:
-        ax = scratch.get('efitviewer_ax', None)
+        ax = scratch.get('efitviewer_cx_ax', None)
     if ax is None:
         ax = gca()
 
@@ -466,7 +491,7 @@ def plot_beam_emission_spectroscopy_overlay(ax=None, ods=None, **kw):
     shot = ods['dataset_description.data_entry'].get('.pulse', evalExpr(settings['cases'][0]['shot']))
 
     if ax is None:
-        ax = scratch.get('efitviewer_ax', None)
+        ax = scratch.get('efitviewer_cx_ax', None)
     if ax is None:
         ax = gca()
 
@@ -500,7 +525,7 @@ def plot_custom_script_overlay(ax=None, ods=None, defaultvars_keywords=None, **k
 
     :param ax: Axes instance [optional]
         If not provided, the custom script had better have a way of choosing.
-        The default axes when using the efitviewer GUI are referenced in scratch.get('efitviewer_ax', None)
+        The default axes when using the efitviewer GUI are referenced in scratch.get('efitviewer_cx_ax', None)
 
     :param ods: ODS instance
 
@@ -555,7 +580,7 @@ def plot_alt_limiter_overlay(ax=None, **kw):
 
     # Set up plot
     if ax is None:
-        ax = scratch.get('efitviewer_ax', None)
+        ax = scratch.get('efitviewer_cx_ax', None)
     if ax is None:
         ax = gca()
     kw.setdefault('label', 'Alternative limiter {}'.format(lim_name))
@@ -750,30 +775,49 @@ def setup_special_contours(case=0, t=None, contour_quantity=None, spacing=None,
     return np.round(contour_levels, decimal_places)
 
 
-def get_axes(ax=None):
+def get_axes(ax=None, which='cx'):
     """
     Choose the best plot axes to use
 
-    :param ax: Axes instance [optional]
+    :param ax: Axes instance or 2D array of Axes instances [optional]
         If not specified, try to use efitviewer GUI's embedded axes. If those are not active, use gca().
 
-    :return: Axes instance
+    :param which: str
+        cx: deal with cross section axes
+        profile: deal with array of profile axes
+
+    :return: Axes instance or 2D array of Axes instances
     """
     if ax is None:
         out_loc, settings_loc, out, settings, parent_settings, scratch = pick_output_location()
 
         printd('plot_single_cx_ods(): trying to look up efitviewer GUI figure...', topic='efitviewer')
-        ax = scratch.get('efitviewer_ax', None)
-    if ax is not None:
+        if which == 'profile':
+            ax = scratch.get('efitviewer_profile_axs', None)
+            if ax is not None and len(ax.flatten()):
+                test_ax = ax[0, 0]
+            else:
+                test_ax = None
+        else:
+            ax = scratch.get('efitviewer_cx_ax', None)
+            test_ax = ax
+    else:
+        test_ax = ax
+        if which == 'profile':
+            if len(test_ax.flatten()):
+                test_ax = ax[0, 0]
+            else:
+                test_ax = None
+    if test_ax is not None:
         # Check if ax is active
         try:
             # https://stackoverflow.com/a/15311949/6605826
-            tk_figure_open = ax.figure.canvas.get_tk_widget().winfo_exists()
+            tk_figure_open = test_ax.figure.canvas.get_tk_widget().winfo_exists()
         except Exception:
             # I assume that get_tk_widget() will fail if the backend isn't tk
             tk_figure_open = False
         # https://stackoverflow.com/a/26485683/6605826
-        figure_window_open = ax.figure in list(map(pyplot.figure, pyplot.get_fignums()))
+        figure_window_open = test_ax.figure in list(map(pyplot.figure, pyplot.get_fignums()))
 
         if (not tk_figure_open) and (not figure_window_open):
             printd('plot_single_cx_ods(): efitviewer figure seems to have been closed', topic='efitviewer')
@@ -781,9 +825,38 @@ def get_axes(ax=None):
     if ax is None:
         printd('plot_single_cx_ods(): using gca() to find axes', topic='efitviewer')
         ax = pyplot.gca()
+        if which == 'profile':
+            # Not sure if this is the best solution, but it should work for now
+            ax_ = np.empty((1, 1), object)
+            ax_[0, 0] = ax
+            ax = ax_
     return ax
 
 
+def save_subplot_layout(reset=False):
+    """
+    Either saves the current layout of subplots in the efitviewer figure, or erases prior records of fig layout
+
+    The saves are particular to each configuration (number of columns, CX on/off, number of profile plots).
+
+    :param reset: bool
+        Erase the last record instead of saving
+    """
+    out_loc, settings_loc, out, settings, parent_settings, scratch = pick_output_location()
+    show_cx = settings.get('show_cx', True)
+    ncol = settings.get('profiles', {}).get('num_profile_cols', 0)
+    if ncol == 0:
+        ncol = 1
+    tag = f'{"cx" if show_cx else ""}-{ncol}-{settings.get("profiles", {}).get("num_profiles", 0)}'
+    if reset:
+        settings['plot_style_kw'].setdefault('subplot_layout', SettingsName()).pop(tag, None)
+        return
+    layout = settings['plot_style_kw'].setdefault('subplot_layout', SettingsName()).setdefault(tag, SettingsName())
+    fig = scratch['efitviewer_fig']
+    for attr in ['wspace', 'hspace', 'left', 'right', 'bottom', 'top']:
+        layout[attr] = getattr(fig.subplotpars, attr, None)
+
+
 def plot_style_enforcement(ax=None, plot_style=None):
     """
     Applies efitviewers plot style settings to some axes
@@ -831,6 +904,15 @@ def plot_style_enforcement(ax=None, plot_style=None):
     # Grid
     plot_grid(fig=fig, ax=ax, enable=plot_style.setdefault('grid_enable', False), grid_kw=plot_style.setdefault('grid_kw', {}))
 
+    # Subplots layout
+    show_cx = settings.get('show_cx', True)
+    ncol = settings.get('profiles', {}).get('num_profile_cols', 0)
+    if ncol == 0:
+        ncol = 1
+    tag = f'{"cx" if show_cx else ""}-{ncol}-{settings.get("profiles", {}).get("num_profiles", 0)}'
+    layout = settings['plot_style_kw'].setdefault('subplot_layout', SettingsName()).setdefault(tag, SettingsName())
+    fig.subplots_adjust(**layout)
+
     # Annotations
     if plot_style.setdefault('subplot_label', None) is not None:
         font_size = settings.get('default_fontsize', None)
@@ -847,6 +929,94 @@ def plot_style_enforcement(ax=None, plot_style=None):
 
 
 # Main plotting functions -----------------------------------------------------------------------------------------
+def populate_efitviewer_figure(
+    fig, scratch, show_cx=True, num_profiles=0, num_profile_cols=0, profiles_sharex='all', profiles_sharey='none', **gridspec_kw
+):
+    """
+    Adds subplots to the efitviewer figure
+
+    :param fig: Figure instance
+        Can be a standalone matplotlib figure, or an OMFITx.Figure GUI element
+
+    :param scratch: dict-like
+        Scratch space
+
+    :param show_cx: bool
+        Show the cross section subplot (otherwise just profiles are shown)
+
+    :param num_profiles: int
+        Number of panels for profile plots
+
+    :param num_profile_cols: int
+        Number of columns for profiles
+
+    :param profiles_sharex: str
+        Share the x axis between profiles. Options: 'none', 'all', 'row', 'col'.
+
+    :param profiles_sharey: str
+        Share the Y axis between profiles. Options: 'none', 'all', 'row', 'col'.
+
+    :param gridspec_kw: keywords to pass to gridspec.
+        width_ratios is especially useful; use it to set the relative widths of
+        columns by passing a numeric iterable matching the total number of
+        columns (including CX, if shown)
+
+    :return: tuple
+        Axes instance: axes for the cross section or None if not used
+        2D array of Axes instances, padded with None as needed: axes for the profiles
+            Padding with None would be needed for cases like 8 plots in a 3x3 arrangement,
+            where one panel would be blank.
+    """
+    # Set up the grid
+    if num_profiles > 0:
+        num_profile_cols_ = np.min([np.max([1, num_profile_cols]), num_profiles])
+        num_rows = int(np.max([np.ceil(num_profiles / float(num_profile_cols_)), 1]))
+    else:
+        num_profile_cols_ = 0
+        num_rows = 1
+    try:
+        gs = matplotlib.gridspec.GridSpec(num_rows, num_profile_cols_ + show_cx, **gridspec_kw)
+    except Exception:
+        gs = matplotlib.gridspec.GridSpec(num_rows, num_profile_cols_ + show_cx)
+        report = ''.join(traceback.format_exception(*sys.exc_info()))
+        printe('Gridspec keywords for setting up efitviewer profile plots were invalid and were ignored!')
+        print(report)
+        printe('Gridspec keywords for setting up efitviewer profile plots were invalid and were ignored!')
+
+    if show_cx:
+        cx_ax = scratch['efitviewer_cx_ax'] = fig.add_subplot(gs[:, 0])
+    else:
+        cx_ax = scratch['efitviewer_cx_ax'] = None
+
+    profile_axs = scratch['efitviewer_profile_axs'] = np.empty((num_rows, num_profile_cols_), dtype=object)
+    profile_axs[:] = None
+    for i in range(num_profiles):
+        row = i % num_rows
+        col = i // num_rows
+        fig_col = col + show_cx
+        if (i > 0) and profiles_sharex != 'none':
+            if profiles_sharex == 'all':
+                sharex = profile_axs[0, 0]
+            elif profiles_sharex == 'row':
+                sharex = profile_axs[row, 0]
+            elif profiles_sharex == 'col':
+                sharex = profile_axs[0, col]
+        else:
+            sharex = None
+        if (i > 0) and profiles_sharey != 'none':
+            if profiles_sharey == 'all':
+                sharey = profile_axs[0, 0]
+            elif profiles_sharey == 'row':
+                sharey = profile_axs[row, 0]
+            elif profiles_sharey == 'col':
+                sharey = profile_axs[0, col]
+        else:
+            sharey = None
+        profile_axs[row, col] = fig.add_subplot(gs[row, fig_col], sharex=sharex, sharey=sharey)
+
+    return cx_ax, profile_axs
+
+
 def plot_cx_ods(ods_idx=None, odss=None, cases=None, **kw):
     r"""
     Wrapper for making multiple calls to plot_single_cx_ods()
@@ -928,8 +1098,61 @@ def plot_cx_ods(ods_idx=None, odss=None, cases=None, **kw):
     return
 
 
+def plot_single_time_profile_set(ods=None, t_idx=0, profile_axs=None, profile_x_quantity=None, profile_y_quantities=None, gentle=False):
+    """
+    Plots profiles for a single time-slice
+
+    :param ods: ODS() instance with the data
+
+    :param t_idx: int
+
+    :param profile_axs: 2D array of Axes instances
+
+    :param profile_x_quantity: str
+
+    :param profile_y_quantities: list of strings
+
+    :param gentle: bool
+        Report exceptions with print and ax.text instead of raising
+    """
+    if profile_axs is None or profile_x_quantity is None:
+        return
+    profiles_1d = ods['equilibrium']['time_slice'][t_idx]['profiles_1d']
+    global_quantities = ods['equilibrium.time_slice'][t_idx]['global_quantities']
+
+    def get_quantity(quantity):
+        """Intercept quantity request to calculate special quantities as needed"""
+        if quantity == 'psi_norm':
+            q = (profiles_1d['psi'] - global_quantities['psi_axis']) / (global_quantities['psi_boundary'] - global_quantities['psi_axis'])
+        else:
+            q = profiles_1d[quantity]
+        return q
+
+    for i, ax in enumerate(profile_axs.flatten()):
+        if profile_y_quantities[i] is None:
+            return
+        ax.set_xlabel(omas_eq1d_pretty_name(profile_x_quantity))
+        ax.set_ylabel(omas_eq1d_pretty_name(profile_y_quantities[i]))
+        try:
+            ax.plot(get_quantity(profile_x_quantity), get_quantity(profile_y_quantities[i]))
+        except (KeyError, ValueError):
+            if gentle:
+                # Display the error instead of bringing the whole GUI down if the plot fails.
+                print('Error in plot_single_time_profile_set:')
+                report = ''.join(traceback.format_exception(*sys.exc_info()))
+                printe(report)
+                # noinspection PyBroadException
+                try:
+                    ax.text(0.05, 0.95, report, ha='left', va='top', color='red')
+                except Exception:
+                    printd('plot_single_cx_ods{}: Failed to properly report an exception!', topic='efitviewer')
+            else:
+                raise
+
+
 def plot_single_cx_ods(
-    ax=None,
+    cx_ax=None,
+    profile_axs=None,
     ods_idx=0,
     multi_cases=False,
     ods=None,
@@ -950,6 +1173,8 @@ def plot_single_cx_ods(
     show_cornernote=None,
     eq_active=None,
     wall_active=None,
+    profile_x_quantity=None,
+    profile_y_quantities=None,
     plot_style_kw=None,
     eqkw=None,
     xkw=None,
@@ -959,8 +1184,13 @@ def plot_single_cx_ods(
 
     If data for an overlay are missing, they will be added before plotting
 
-    :param ax: Axes instance
-        Axes to draw on. Defaults to gca() if not specified.
+    :param cx_ax: Axes instance
+        Axes to draw cross section on. Defaults to gca() if not specified.
+
+    :param profile_axs: 2D array containing Axes instances
+        Axes to use for drawing profiles.
+        Profiles disabled if set to None
+        Pad with None if necessary (e.g. 8 plots in a 3x3 arrangement with one missing).
 
     :param ods_idx: int
         For looking up which group of settings to use, & for looking up which ODS to use, if ods is not overridden.
@@ -1031,6 +1261,12 @@ def plot_single_cx_ods(
     :param wall_active: bool
         Wall overlay active / included in eq overlay?
 
+    :param profile_x_quantity: str
+        Quantity to use on the X-axes of the profile plots.
+
+    :param profile_y_quantities: list of strings
+        Quantities to show on the Y-axes of the profile plots. Must be at least as long as profile_axs.
+
     :param plot_style_kw: dict-like
         Settings for enforcing plot axes style
 
@@ -1063,10 +1299,15 @@ def plot_single_cx_ods(
     if timing_ref is not None:
         printe('-' * 80)
 
-    ax = get_axes(ax)
-    fig = ax.figure
+    cx_ax = get_axes(cx_ax)
+    fig = cx_ax.figure
+    profile_axs = get_axes(profile_axs, which='profile')
     if clear_first:
-        ax.cla()
+        if cx_ax is not None:
+            cx_ax.cla()
+        if profile_axs is not None:
+            for ax in profile_axs.flatten():
+                ax.cla()
 
     # noinspection PyBroadException
     try:
@@ -1110,6 +1351,11 @@ def plot_single_cx_ods(
             eq_active = settings['cases'][ods_idx]['active']
         if wall_active is None:
             wall_active = settings['cases'][ods_idx].get('wall', True)
+        settings.setdefault('profiles', SettingsName())
+        if profile_x_quantity is None:
+            profile_x_quantity = settings['profiles'].get('xaxis_quantity', None)
+        if profile_y_quantities is None:
+            profile_y_quantities = [settings['profiles'].get(f'yaxis_quantity_{i}', None) for i in range(len(profile_axs.flatten()))]
         if plot_style_kw is None:
             plot_style_kw = settings.setdefault('plot_style_kw', {})
         if eqkw is None:
@@ -1164,7 +1410,7 @@ def plot_single_cx_ods(
                     eqcx_label = 'Equilibrium {:0.1f} ms'.format(ods['equilibrium.time'][it] * 1000)
                 eqcx_label = eqkw.pop('label', eqcx_label)
                 eqcxkw = dict(
-                    ax=ax,
+                    ax=cx_ax,
                     time_index=it,
                     levels=levels,
                     label=eqcx_label,
@@ -1185,13 +1431,21 @@ def plot_single_cx_ods(
                         eqcxkw.pop(kw, None)
 
                 ods.plot_equilibrium_CX(**eqcxkw)  # The call to draw the equilibrium contours
+                plot_single_time_profile_set(
+                    ods=ods,
+                    t_idx=it,
+                    profile_axs=profile_axs,
+                    profile_x_quantity=profile_x_quantity,
+                    profile_y_quantities=profile_y_quantities,
+                    gentle=gentle,
+                )
         else:
             printd('plot_single_cx_ods{}: this eq slice is INACTIVE; skipping eq plot', topic='efitviewer')
             it = 0
-            ax.set_aspect('equal')
-            ax.set_frame_on(False)
-            ax.xaxis.set_ticks_position('bottom')
-            ax.yaxis.set_ticks_position('left')
+            cx_ax.set_aspect('equal')
+            cx_ax.set_frame_on(False)
+            cx_ax.xaxis.set_ticks_position('bottom')
+            cx_ax.yaxis.set_ticks_position('left')
 
         def overlay_customizations(ovls):
             ovls = dict(ovls)
@@ -1232,7 +1486,7 @@ def plot_single_cx_ods(
         scratch['overlays'] = overlays
         # Thomson scattering has a different default than others, so we must make sure blank is replaced by False
         overlays.setdefault('thomson_scattering', False)
-        ods.plot_overlay(ax=ax, **overlays)  # The call to do the overlays
+        ods.plot_overlay(ax=cx_ax, **overlays)  # The call to do the overlays
 
         # Plot special hardware overlay(s)
         special_overlays = overlay_customizations(special_overlays)
@@ -1246,28 +1500,28 @@ def plot_single_cx_ods(
                 except NameError:
                     printe('Unrecognized special overlay: {} has no function (expected {})'.format(so, function_name))
                 else:
-                    special_function(ax=ax, ods=ods, **skw)
+                    special_function(ax=cx_ax, ods=ods, **skw)
 
         # Annotations and finishing touches
         actually_show_legend = (multi_cases and (show_legend > 0)) or (show_legend == 1)
         actually_show_cornernote = (not multi_cases and (show_cornernote > 0)) or (show_cornernote == 1)
         if actually_show_legend:
-            ax.legend(loc=0)
+            cx_ax.legend(loc=0)
         if actually_show_cornernote:
             cornernote(
-                ax=ax,
+                ax=cx_ax,
                 device=device,
                 shot='{} {}'.format(shot, efitid),
                 time='{:0.2f}'.format(ods['equilibrium.time'][it] * 1000) if len(t) == 1 else ' ',
             )
         if xlim is not None:
-            ax.set_xlim(xlim)
+            cx_ax.set_xlim(xlim)
         if ylim is not None:
-            ax.set_ylim(ylim)
+            cx_ax.set_ylim(ylim)
         if len(t) > 1:
-            ax.legend(loc=3)
+            cx_ax.legend(loc=3)
 
-        plot_style_enforcement(ax=ax, plot_style=plot_style_kw)
+        plot_style_enforcement(ax=cx_ax, plot_style=plot_style_kw)
 
         if default_fontsize is not None:
             rcParams['font.size'] = old_font_size
@@ -1286,11 +1540,11 @@ def plot_single_cx_ods(
             printe(report)
             # noinspection PyBroadException
             try:
-                if ax is None:
-                    ax = gca()
+                if cx_ax is None:
+                    cx_ax = gca()
                 if clear_first:
-                    ax.cla()
-                ax.text(0.05, 0.95, report, ha='left', va='top', color='red')
+                    cx_ax.cla()
+                cx_ax.text(0.05, 0.95, report, ha='left', va='top', color='red')
             except Exception:
                 printd('plot_single_cx_ods{}: Failed to properly report an exception!', topic='efitviewer')
         else:
@@ -1341,6 +1595,8 @@ def dump_plot_command_content():
     for index in settings['cases']:
         load_instructions_string = load_instructions_string.replace("'replace_{}'".format(index), load_instructions_dict[index])
 
+    _, default_profile_y_quantities = find_profile_y_options()
+
     contents = '''def plot_eq_cx():
     """
     Plots an equilibrium slice with a specific configuration of hardware overlay(s)
@@ -1351,16 +1607,28 @@ def dump_plot_command_content():
     OMFIT version: {omfit_version:}
     """
 
-    from omfit_classes.omfit_efitviewer import plot_cx_ods
+    from omfit_classes.omfit_efitviewer import plot_cx_ods, populate_efitviewer_figure
     device = {device:}
     shot = {shot:}
     cases = {cases:}
     odss = {load_instructions:}
-    fig, ax = pyplot.subplots(1)
+    fig = pyplot.figure()
+    gridspec_kw = {gridspec_kw:}
+    cx_ax, profile_axs = populate_efitviewer_figure(
+        fig,
+        scratch,
+        show_cx={show_cx:},
+        num_profiles={num_profiles:},
+        num_profile_cols={num_profile_cols:},
+        profiles_sharex={profiles_sharex:},
+        profiles_sharey={profiles_sharey:},
+        **gridspec_kw
+        )
     levels = {levels:}
     customization = {customization:}
     plot_cx_ods(
-        ax=ax,
+        cx_ax=cx_ax,
+        profile_axs=profile_axs,
         odss=odss,
         cases=cases,
         contour_quantity={contour_quantity:},
@@ -1375,6 +1643,8 @@ def dump_plot_command_content():
         customization=customization,
         default_fontsize={default_fontsize:},
         show_legend={show_legend:},
+        profile_x_quantity={profile_x_quantity:},
+        profile_y_quantities={profile_y_quantities:},
     )
     return
 
@@ -1399,6 +1669,17 @@ def dump_plot_command_content():
         omfit_version=repo.active_branch()[-1],
         default_fontsize=default_fontsize,
         show_legend=settings.setdefault('show_legend', 2),
+        gridspec_kw=repr(settings['profiles'].get('gridspec_kw', {})),
+        show_cx=settings.get('show_cx', True),
+        num_profiles=settings['profiles'].get('num_profiles', 0),
+        num_profile_cols=settings['profiles'].get('num_profile_cols', 0),
+        profiles_sharex=repr(settings['profiles'].get('sharex', 'all')),
+        profiles_sharey=repr(settings['profiles'].get('sharey', 'none')),
+        profile_x_quantity=repr(settings['profiles'].get('xaxis_quantity', None)),
+        profile_y_quantities=[
+            settings['profiles'].get(f'yaxis_quantity_{i}', default_profile_y_quantities[i])
+            for i in range(settings['profiles'].get('num_profiles', 0))
+        ],
     )
     contents += '\nplot_eq_cx()'
     contents = omfit_formatter(contents)
@@ -1601,7 +1882,7 @@ def grab_xylim(decimals=None):
             return a
         return tuple([np.round(aa, decimals) if is_numeric(aa) else aa for aa in np.atleast_1d(a)])
 
-    ax = scratch.get('efitviewer_ax', None)
+    ax = scratch.get('efitviewer_cx_ax', None)
     if ax is not None:
         xlim = proccess_lim(ax.get_xlim())
         ylim = proccess_lim(ax.get_ylim())
@@ -1641,3 +1922,53 @@ def get_default_snap_list(device):
         if is_device(device, gd):
             return guesses[gd]
     return {}
+
+
+def find_cq_options():
+    """
+    Determines what should be available as options for the contour quantity
+
+    :return: tuple
+        list: list of options (as strings)
+        str: name of the recommended default contour quantity
+
+    """
+    out_loc, settings_loc, out, settings, parent_settings, scratch = pick_output_location()
+    cq_opts = []
+    if 'psi' in out['efitviewer_ods']['equilibrium.time_slice[0].profiles_2d[0]']:
+        cq_opts += ['psi_norm', 'psi']
+        if 'psi' in out['efitviewer_ods']['equilibrium.time_slice[0].profiles_1d']:
+            # psi is used to interpolate, so 1d and 2d psi are both needed to support the 1d profiles.
+            # Here are some popular ones:
+            popular_from_1d = ['q', 'rho_tor', 'rho_tor_norm']
+            for pf1 in popular_from_1d:
+                if pf1 in out['efitviewer_ods']['equilibrium.time_slice[0].profiles_1d']:
+                    cq_opts += [pf1]
+    if 'phi' in out['efitviewer_ods']['equilibrium.time_slice[0].profiles_2d[0]']:
+        cq_opts += ['phi']
+    if len(cq_opts):
+        default_contour_quantity = 'psi_norm' if 'psi_norm' in cq_opts else cq_opts[0]
+    else:
+        default_contour_quantity = None
+    return cq_opts, default_contour_quantity
+
+
+def find_profile_y_options(index=0):
+    """
+    Finds a list of quantities that could go on the y axis of the profile plots
+
+    :param index: int
+        Index of time-slice to check in, in case different times have different availability (hopefully not)
+
+    :return: tuple
+        list: all options for y axis quantities
+        list: list of recommended defaults for each plot panel
+    """
+    out_loc, settings_loc, out, settings, parent_settings, scratch = pick_output_location()
+    yopts = [k for k, v in out['efitviewer_ods']['equilibrium']['time_slice'][index]['profiles_1d'].items() if isinstance(v, np.ndarray)]
+    priorities = ['pressure', 'q', 'dpressure_dpsi', 'f_df_dpsi', 'phi']
+    ydefault = [p for p in priorities if p in yopts]
+    ydefault += [y for y in yopts if y not in ydefault]
+    ydefault += yopts
+    ydefault += [None] * 100
+    return yopts, ydefault
