@@ -51,6 +51,8 @@ __all__ = [
     'solveMDSserver',
     'OMFITmdsCache',
     'interpret_signal',
+    'available_efits_from_mds',
+    'check_efit_tree_validity',
 ]
 
 D3D_mds_server_trename_selector = {'': 'atlas', 'lbo': 'orion'}
@@ -110,6 +112,8 @@ _MDSserverDict = {
     'AUG': 'aug_mdsplus',
     'ASDEX': 'aug_mdsplus',
     'aug_mdsplus': 'aug_mdsplus',
+    #
+    'ST40': 'smaug',
 }
 
 # from MainSettings['SERVER'] server to MDS+ top tree name
@@ -124,6 +128,7 @@ _topLevelMDStreename = {
     'mst_mdsplus': 'MST',
     'aug_mdsplus': 'ASDEX',
     'transpgrid': 'TRANSP',
+    'smaug': 'ST40',
 }
 
 my_ip_address = [get_ip()]
@@ -809,7 +814,13 @@ class OMFITmdsConnectionBaseClass(SortedDict):
                 txt = 'build_with_error(%s,%s)' % (txt, txt)
                 arg.extend([nominal_values(item.values), std_devs(item.values)])
             else:
-                arg.append(item.values)
+                if item.values.dtype == 'object':
+                    printw(f"\nCasting {dim} to float array: It has dtype='object' but is_uncertain == False")
+                    # OMFITprofiles have scripts that creates dtype='object' DataArrays with no uncertainty.
+                    # Need to cast to float
+                    arg.append(item.values.astype(float))
+                else:
+                    arg.append(item.values)
             try:
                 conn.put(f'{translator(dim)}', txt, *arg)
             except Exception as _excp:
@@ -817,6 +828,7 @@ class OMFITmdsConnectionBaseClass(SortedDict):
 
         # subselect list of quantities from variables
         var_quantities = list(xarray_dataset.data_vars.keys())
+        # note for future: omfit_profiles depends on var_quantities being ordered! (or the coords and units it gets back will be all jumbled).
         if quantities is not None:
             var_quantities = [k for k in var_quantities if k in quantities]
         # assign variables
@@ -830,7 +842,12 @@ class OMFITmdsConnectionBaseClass(SortedDict):
                 txt = 'build_with_error(%s,%s)' % (txt, txt)
                 arg.extend([nominal_values(item.values), std_devs(item.values)])
             else:
-                arg.append(item.values)
+                if item.values.dtype == 'object':
+                    printw(f"\nCasting {var} to float array: It has dtype='object' but is_uncertain == False")
+                    arg.append(item.values.astype(float))
+                else:
+                    arg.append(item.values)
+
             txt = 'make_signal(%s,*,%s)' % (txt, ','.join([translator(dim) for dim in reversed(item.dims)]))
             try:
                 conn.put(f'{translator(var)}', txt, *arg)
@@ -1625,6 +1642,147 @@ class OMFITmdsValue(dict, OMFITmdsObjects):
     # End of class OMFITmdsValue
 
 
+def check_efit_tree_validity(device, shot, treename):
+    """
+    Checks whether an EFIT tree is populated / appears to have valid results
+
+    :param device: str
+        Device name
+
+    :param shot: int
+        Shot number
+
+    :param treename: str
+        Name of the MDSplus tree for the specific EFIT result in question, like 'EFIT01'
+
+    :return: bool
+        True if the tree might be okay, False if the tree is not okay
+    """
+    try:
+        printd(f'check {device}#{shot} {treename}', topic='available_EFITs')
+        mds = OMFITmdsValue(device, shot=shot, treename=treename, TDI=rf'GETNCI("\\{treename}::TOP.RESULTS.GEQDSK.GTIME", "length")')
+        if mds.data() is None:
+            return False
+        return mds.data()[0] > 0
+    except MDSplus.mdsExceptions.TreeFILE_NOT_FOUND:
+        return False
+
+
+def available_efits_from_mds(scratch_area, device, shot, default_snap_list=None, format='{tree}', no_empty=True, check_contents=False):
+    """
+    Attempts to lookup a list of available EFITs from MDSplus
+
+    Works for devices that store EFITs together in a group under a parent tree, such as:
+    EFIT            (parent tree)
+      |- EFIT01     (results from an EFIT run)
+      |- EFIT02     (results from another run)
+      |- EFIT03
+      |- ...
+
+    If the device's MDSplus tree is not arranged like this, it will fail and return [].
+
+    The number of MDSplus calls made by this function is determined by:
+        no_empty = False, check_contents = False: 1 call
+        no_empty =  True, check_contents = False: 2 calls
+        no_empty = False, check_contents =  True: 1 + N calls
+        no_empty =  True, check_contents =  True: 2 + N calls
+    Where N is the number of EFIT results found.
+
+    :param scratch_area: dict
+        Scratch area for storing results to reduce repeat calls. Mainly included to match
+        call sigure of available_efits_from_rdb(), since OMFITmdsValue already has caching.
+
+    :param device: str
+        Device name
+
+    :param shot: int
+        Shot number
+
+    :param default_snap_list: dict [optional]
+        Default set of EFIT treenames. Newly discovered ones will be added to the list.
+
+    :param format: str
+        Instructions for formatting data to make the EFIT tag name.
+        Provided for compatibility with available_efits_from_rdb() because the only option is '{tree}'.
+
+    :param no_empty: bool
+        Use a second TDI command to check the number of nodes in each EFIT and
+        filter out EFIT trees that are empty (0 nodes within them).
+        Adds one MDSplus call.
+
+    :param check_contents: bool
+        Do a deeper check to make sure GEQDSK data are populated with at least one time.
+        Adds one MDSplus call per EFIT that was found so far.
+
+    :return: (dict, str)
+        Dictionary keys will be descriptions of the EFITs
+            Dictionary values will be the formatted identifiers.
+            For now, the only supported format is just the treename.
+            If lookup fails, the dictionary will be {'': ''} or will only contain default results, if any.
+        String will contain information about the discovered EFITs
+    """
+    # Basic setup
+    printd(f'Searching for available EFITs in MDS for device = {device}, shot = {shot}', topic='available_EFITs')
+    if default_snap_list is None:
+        default_snap_list = {'': ''}
+    snap_list = copy.deepcopy(default_snap_list)
+    help_info = ''
+    scratch_area.setdefault('searched_in', [])
+    scratch_area['searched_in'] += ['mds']
+
+    efit_group_parent_treename = {  # None means that EFITs for this device are not organized under a single parent
+        'DIII-D': None,
+        'EAST': None,
+        'KSTAR': 'EFIT',
+        'NSTX': 'EFIT',
+    }
+    efit_group_parent_treename['NSTX-U'] = efit_group_parent_treename['NSTX']
+    treename = efit_group_parent_treename.get(tokamak(device), None)
+
+    # Complain about problems
+    if treename is None:
+        printd(f"  No rule for looking up all EFITs from MDSplus for {device}.", topic='available_EFITs')
+        scratch_area['available_efits_from_mds_success'] = False
+        return snap_list, help_info
+    else:
+        printd(f'  EFITs for {device} should be stored in treename = {treename}.', topic='available_EFITs')
+
+    if format != '{tree}':
+        raise NotImplementedError(
+            'Custom formatting of available EFIT results is not available when searching for EFITs in MDSplus. '
+            'Please use format="{tree}" intead.'
+        )
+
+    # Try to gather information
+    mds = OMFITmdsValue(device, shot=shot, treename=treename, TDI=rf'GETNCI("\\{treename}::TOP.*", "MINPATH")')
+    if mds.check():
+        efits = [d.split('.')[-1] for d in mds.data()]
+        help_info = f'Found list of EFITs in MDSplus for {device}#{shot}.'
+        # Options to filter the list
+        if no_empty:
+            tdi2 = rf'GETNCI("\\{treename}::TOP.*", "number_of_members")'
+            mds2 = OMFITmdsValue(device, shot=shot, treename=treename, TDI=tdi2)
+            validity = [d > 0 for d in mds2.data()]
+            if len(validity) == len(efits):
+                efits = list(array(efits)[array(validity)])
+            help_info += ' Removed EFITs that were totally empty.'
+        if check_contents:
+            validity2 = [check_efit_tree_validity(device, shot, efit) for efit in efits]
+            efits = list(array(efits)[array(validity2)])
+            help_info += ' Removed EFITs that had no GEQDSK time slices.'
+        printd(f'  Found EFITs in MDSplus for {device}: {efits}', topic='available_EFITs')
+        # Format the results
+        for tree in efits:
+            snap_list[f"[{tree}]"] = format.format(tree=tree)
+        scratch_area['available_efits_from_mds_success'] = True
+        return snap_list, help_info
+    else:
+        printd(f'  Failed to lookup list of EFITs in MDSplus for {device}#{shot}.', topic='available_EFITs')
+        scratch_area['available_efits_from_mds_success'] = False
+        help_info = f'Failed to locate any EFITs in MDSplus for {device}#{shot}'
+        return snap_list, help_info
+
+
 # ==================
 # Signal interpreter
 # ==================
