@@ -33,15 +33,15 @@ def process_OMFITexpression(value):
 try:
     omas.omas_core.input_data_process_functions.append(process_OMFITexpression)
 except AttributeError:
-    # avoid OMFIT crashing if OMAS version is not the latest
-    pass
+    # avoid OMFIT not starting if OMAS version is not the latest
+    printe(f'WARNING: Your OMAS installation {os.path.dirname(omas.__file__)} is old. Please update it.')
 
 # hook up omfit and omas print streams
 omas.omas_utils._streams = _streams
 
 
 def tunnel_mds(server, treename):
-    '''
+    """
     Resolve MDS+ server
     NOTE: This function makes use of the optional `omfit_classes` dependency to establish a SSH tunnel to the MDS+ server.
 
@@ -50,7 +50,7 @@ def tunnel_mds(server, treename):
     :param treename: treename (in case treename affects server to be used)
 
     :return: string with MDS+ server and port to be used
-    '''
+    """
     try:
         import omfit_classes.omfit_mds
     except (ImportError, ModuleNotFoundError):
@@ -63,18 +63,22 @@ def tunnel_mds(server, treename):
     return server.format(**os.environ)
 
 
-omas.omas_machine.tunnel_mds = tunnel_mds
+try:
+    omas.omas_machine.tunnel_mds = tunnel_mds
+except AttributeError:
+    # avoid OMFIT not starting if OMAS version is not the latest
+    printe(f'WARNING: Your OMAS installation {os.path.dirname(omas.__file__)} is old. Please update it.')
 
 # export __all__ from omas.__all__
 __all__ = list([x for x in ['omas'] + list(omas.__dict__.keys()) if not x.startswith('_')])
 
 
 def pprint_imas_data_dictionary_info(location):
-    '''
+    """
     pretty print IMAS data dictionary info
 
     :param location: location in IMAS data dictionary
-    '''
+    """
     location = omas.omas_utils.p2l(location)
     location = omas.omas_utils.l2u(location)
     info = omas_info_node(location.rstrip('.:'))
@@ -361,7 +365,7 @@ class OMFITiterscenario(OMFITjson):
         return '\n'.join(output)
 
     def filter(self, conditions, squash=False):
-        '''
+        """
         Filter database for certain conditions
 
         :param conditions: dictionary with conditions for returning a match. For example:
@@ -370,7 +374,7 @@ class OMFITiterscenario(OMFITjson):
         :param squash: remove attributes that are equal among all entries
 
         :return: OMFITiterscenario dictionary only with matching entries
-        '''
+        """
         matches = OMFITiterscenario('filtered.json')
 
         for item in self:
@@ -406,7 +410,7 @@ class OMFITiterscenario(OMFITjson):
 
 
 def iter_scenario_summary_remote(quiet=False, environment="module purge\nmodule load IMAS"):
-    '''
+    """
     Access iter server and sun `scenario_summary` command
 
     :param quiet: print output of scenario_summary to screen or not
@@ -414,7 +418,7 @@ def iter_scenario_summary_remote(quiet=False, environment="module purge\nmodule
     :param environment: `module load {IMAS_module}` or the like
 
     :return: dictionary with info from available scenarios
-    '''
+    """
 
     # fields to be read
     # 'composition',
