@@ -546,6 +546,47 @@ class OMFITpFile(SortedDict, OMFITascii):
             pyplot.draw()
             pyplot.ion()
 
+    def remap(self, points='ne', **kw):
+        """
+        Remap the disparate psinorm grids for each vaiable onto the same psinorm grid
+
+        :param npoints: number of points to remap the originally 256 grid
+            - If points is int: make an evenly spaced array.
+            - If points is array: use this as the grid
+            - If points is a string: use the 'psinorm' from that item in the pfile (by default `ne['psinorm']`)
+
+        :param \**kw: addidional keywords are passed to scipy.interpolate.interp1d
+
+        return: The entire object remapped to the same psinorm grid
+        """
+
+        mapped = copy.deepcopy(self)
+        keys = mapped.keys()
+
+        if isinstance(points, str):
+            remap_psin = self[points]['psinorm']
+
+        elif type(points) is int:  # Evenly spaced grid rebase
+            remap_psin = np.linspace(0, 1, points)
+
+        elif type(points) in [list, tuple, np.ndarray]:  # user provided grid rebase
+            assert np.all(np.abs(points) <= 1), 'All values must be on the interval [0, 1]'
+            assert np.all(np.diff(points) > 0), 'All values must be monotonically increasing'
+            remap_psin = points
+
+        mapped['psinorm'] = remap_psin  # save the psin grid
+        # iterate through all keys
+        for key in keys:
+            if key in ('N Z A',):  # this is not mapped to psi so skip it
+                continue
+            f_key = interp1d(self[key]['psinorm'], self[key]['data'])
+            f_key_derivative = interp1d(self[key]['psinorm'], self[key]['derivative'], **kw)
+            mapped[key]['psinorm'] = remap_psin
+            mapped[key]['data'] = f_key(remap_psin)
+            mapped[key]['derivative'] = f_key_derivative(remap_psin)
+
+        return mapped
+
     def to_omas(self, ods=None, time_index=0, gEQDSK=None):
         """
         translate OMFITpFile class to OMAS data structure
