@@ -19,6 +19,7 @@ from omfit_classes.omfit_eqdsk import OMFITgeqdsk
 from omfit_classes.omfit_data import OMFITncDataset, OMFITncDynamicDataset, importDataset
 from omas import ODS, omas_environment, cocos_transform, define_cocos
 from omfit_classes.omfit_omas_utils import add_generic_OMFIT_info_to_ods
+from omfit_classes.omfit_rdb import OMFITrdb
 
 import inspect
 import numpy as np
@@ -28,17 +29,21 @@ from scipy import constants
 
 np.seterr(invalid='ignore', divide='ignore')
 
-__all__ = ['OMFITprofiles', 'OMFITprofilesDynamic']
+__all__ = ['OMFITprofiles', 'OMFITprofilesDynamic', 'available_profiles']
 
 model_tree_species = ['e', '2H1', '4He2', '6Li3', '10B5', '12C6', '14N7', '20Ne10']
+
 # fmt: off
 model_tree_quantities = ['angular_momentum', 'angular_momentum_density', 'angular_momentum_density_{species}', 'angular_momentum_{species}',
-'dn_{species}_dpsi', 'dT_{species}_dpsi', 'ELM_phase', 'ELM_since_last', 'ELM_until_next', 'fpol', 'jboot_sauter', 'lnLambda',
-'n_{species}', 'nclass_sigma', 'nu_star_{species}', 'nu_{species}', 'omega_gyro_{species}_midplane', 'omega_LX_{species}_midplane',
-'omega_N_{species}', 'omega_NTV0_{species}', 'omega_P_{species}', 'omega_plasma_{species}', 'omega_RX_{species}_midplane',
-'omega_T_{species}', 'omega_tor_{species}', 'omega_tor_{species}_KDG', 'P_brem', 'P_rad', 'P_rad_cNi', 'P_rad_cW', 'P_rad_nNi', 'P_rad_nW',
-'P_rad_ZnNi', 'P_rad_ZnW', 'p_tot', 'p_{species}', 'pres', 'psi', 'psi_n', 'q', 'R_midplane', 'resistivity', 'rho', 'T_{species}', 'time',
-'Total_Zeff', 'V_pol_{species}_KDG', 'Zavg_Ni', 'Zavg_W', 'Zeff']
+        'dn_{species}_dpsi', 'dT_{species}_dpsi', 'ELM_phase', 'ELM_since_last', 'ELM_until_next', 'Er_{species}_gradP', 'Er_{species}_gradP_Vtor',
+        'Er_{species}_Vtor', 'f_Z', 'fpol', 'gamma_ExB_{species}_gradP', 'gamma_ExB_{species}_Vtor', 'J_BS', 'J_efit_norm', 'J_ohm', 'J_tot',
+        'jboot_sauter', 'lnLambda', 'mass_density', 'n_fast_{species}', 'n_{species}', 'nclass_sigma', 'nu_star_{species}', 'nu_{species}',
+        'omega_gyro_{species}_midplane', 'omega_LX_{species}_midplane', 'omega_N_{species}', 'omega_NTV0_{species}', 'omega_P_{species}',
+        'omega_plasma_{species}', 'omega_RX_{species}_midplane', 'omega_T_{species}', 'omega_tor_{species}', 'omega_tor_{species}_KDG', 'P_brem',
+        'p_fast_{species}', 'P_rad', 'P_rad_cNi', 'P_rad_cW', 'P_rad_int', 'P_rad_nNi', 'P_rad_nW', 'P_rad_ZnNi', 'P_rad_ZnW', 'p_thermal', 'p_tot',
+        'p_total', 'p_{species}', 'pres', 'psi', 'psi_n', 'q', 'R_midplane', 'resistivity', 'rho', 'SAWTOOTH_phase', 'SAWTOOTH_since_last',
+        'SAWTOOTH_until_next', 'sigma_nc', 'T_fast_{species}', 'T_i', 'T_i_T_e_ratio', 'T_{species}', 'time', 'Total_Zeff', 'V_pol_{species}_KDG',
+        'V_tor_{species}', 'Zavg_Ni', 'Zavg_W', 'Zeff']
 # fmt: on
 
 
@@ -102,6 +107,22 @@ def get_species(derived):
     return species, ions, ions_with_dens, ions_with_fast
 
 
+def available_profiles(server, shot, device='DIII-D', verbose=True):
+    out = {}
+    db = OMFITrdb(db='code_rundb', server='d3drdb', by_column=True)
+    runs = db.select(f"SELECT * FROM plasmas WHERE code_name='OMFITprofiles' AND experiment='{device}' AND shot={shot}")
+    if len(runs) == 0:
+        print("No run_id found for this shot.")
+        return out
+    else:
+        for i, runid in enumerate(runs['run_id']):
+            out[
+                runid
+            ] = f"runid={runid} by {runs['run_by'][i]} from {runs['start_time'][i]} to {runs['stop_time'][i]} with comment: {runs['run_comment'][i]}"
+
+    return out
+
+
 class OMFITprofiles(OMFITncDataset):
     """
     Data class used by OMFITprofiles, CAKE and other
@@ -331,37 +352,51 @@ class OMFITprofiles(OMFITncDataset):
         # ods.physics_edge_profiles_pressures() # This function does not exist, but really should.
         return ods
 
-    def model_tree_quantities(self):
+    def model_tree_quantities(self, warn=True, no_update=False, details=False):
         """
         Returns list of MDS+ model_tree_quantities for all species.
-        The function will warn if some of the `model_tree_quantities` are missing in OMFIT-source/omfit/omfit_classes/omfit_profiles.py
-        and the model tree should be updated
+
+        :param warn: [bool] If True, the function will warn if some of the `model_tree_quantities` are missing in
+            OMFIT-source/omfit/omfit_classes/omfit_profiles.py and the model tree should be updated
+
+        :param no_update: [bool] If True, the function will return only items that is in the object AND on the model
+            tree, and ignore items that is not in model_tree_quantities.
 
         :return: list of strings
         """
         new_model_tree_quantities = set(model_tree_quantities)
-        for item in self.variables:
-            match = False
-            for s in model_tree_species:
-                tmp = item.replace(f'_{s}', '_{species}')
-                if tmp != item:
-                    match = True
-                    break
-            if match:
-                new_model_tree_quantities.add(tmp)
-            else:
-                new_model_tree_quantities.add(item)
+        if not no_update:
+            for item in self.variables:
+                match = False
+                dont_replace = ['T_i_T_e_ratio', 'J_efit_norm']  # Don't make this into T_i_T_{species} ratio please
+                for s in model_tree_species:
+                    if item not in dont_replace:
+                        tmp = item.replace(f'_{s}', '_{species}')
+                    if tmp != item:
+                        match = True
+                        break
+                if match:
+                    new_model_tree_quantities.add(tmp)
+                elif not no_update:
+                    new_model_tree_quantities.add(item)
+
         new_model_tree_quantities = sorted(list(new_model_tree_quantities), key=lambda x: x.lower())
-        if new_model_tree_quantities != model_tree_quantities:
+        if new_model_tree_quantities != model_tree_quantities and warn:
             import textwrap
 
-            printe('WARNING!: Update model_tree_quantities in OMFIT-source/omfit/omfit_classes/omfit_profiles.py')
-            printe('WARNING!: and update the OMFIT_PROFS MDS+ model tree')
-            printe('-' * 140)
-            printe('# fmt: off')
-            printe(textwrap.fill(f'model_tree_quantities = {repr(new_model_tree_quantities)}', width=140))
-            printe('# fmt: on')
-            printe('-' * 140)
+            if details:
+
+                printe('WARNING!: Update model_tree_quantities in OMFIT-source/omfit/omfit_classes/omfit_profiles.py')
+                printe('WARNING!: and update the OMFIT_PROFS MDS+ model tree')
+                printe('-' * 140)
+                printe('# fmt: off')
+                printe(textwrap.fill(f'model_tree_quantities = {repr(new_model_tree_quantities)}', width=140))
+                printe('# fmt: on')
+                printe('-' * 140)
+            else:
+                printe("WARNING!: Profile vars mismatch with model tree!")
+                printe("WARNING!: Consider using the 'relaxed' option with to_mds()")
+                printe("WARNING!: Or use .model_tree_quantities(details=True) for instructions to update model tree.")
 
         quantities = []
         for item in new_model_tree_quantities:
@@ -386,9 +421,24 @@ class OMFITprofiles(OMFITncDataset):
 
         quantities = {self.mds_translator(k): None for k in self.model_tree_quantities()}
         quantities['__content__'] = ''
+        quantities['__x_coord__'] = ''
+        quantities['__coords__'] = ''
+        quantities['__dsp_name__'] = ''
+        quantities['__attrs__'] = ''
+        quantities['__comment__'] = ''
         conn.create_model_tree(treename, '', quantities, clear_subtree=True)
 
-    def to_mds(self, server, shot, treename='OMFIT_PROFS'):
+    def check_attrs(self, quiet=False):
+        """
+        Checks that basic/standard attrs are present. If not, they will be fill with standby values (usually 'unknown')
+        Also checks that ints are ints and not int64, which would prevent json from working properly.
+
+        :param quiet: If set to True, the function will not print warnings. By default set to False.
+        """
+
+        basic_atts = ['shot', 'produced_by_module', 'produced_by_user']
+
+    def to_mds(self, server, shot, treename='OMFIT_PROFS', skip_vars=[], comment=None, tag=None, relaxed=False, commit=True):
         """
         This script writes the OMFITproflies datase to DIII-D MDS+ and updates d3drdb accordingly
 
@@ -398,23 +448,70 @@ class OMFITprofiles(OMFITncDataset):
 
         :param treename: MDS+ treename
 
+        :param skip_vars: variables to skip uploading. Array-like
+
+        :param relaxed: if set to True, then the function will only try to upload vars in the model_tree_quantities
+            list as recorded at the beginging of this file. If False, then this funct will attempt to upload all
+            variables stored in self, and fail if a profile variable cannot be uploaded (usually due there not being a
+            corresponding node on the MDS+ tree).
+
+        :param commit (bool): If set to False, the SQL query will not commit the data to the coderunrdb. This is required to be
+            false for a jenkins test or else if it tries to write data to SQL database twice it will throw an error.
+
         :return: runid, treename
         """
-        from omfit_classes.omfit_mds import OMFITmdsConnection
+
+        # Parse comments
+        if comment is None:
+            comment = self.comment
+        else:
+            self.comment = comment  # Update object comment to be consistent
+
+        from omfit_classes.omfit_mds import OMFITmdsConnection, translate_MDSserver
+        import json
+        from omfit_classes.omfit_json import dumper
 
         conn = OMFITmdsConnection(server)
 
-        quantities = self.model_tree_quantities()
+        if relaxed:
+            quantities = self.model_tree_quantities(warn=False, no_update=True)
+        else:
+            quantities = self.model_tree_quantities()
+
+        quantities = [x for x in quantities if x not in skip_vars]
+        # Determine radial coord
+        x_coord = None
+        if 'psi_n' in self.coords.keys() and 'rho' in self.coords.keys():
+            x_coord = 'unclear'
+            # raise Exception("Confusion exist in radial coordinate used. Make sure dataset have a single radial coordinate.")
+        elif 'psi_n' in self.coords.keys():
+            x_coord = 'psi_n'
+        elif 'rho' in self.coords.keys():
+            x_coord = 'rho'
 
         # find next available runid in d3drdb for this shot
         from omfit_classes.omfit_rdb import OMFITrdb
 
         rdb = OMFITrdb(db='code_rundb', server='d3drdb', by_column=True)
-        runs = rdb.select(f"select run_id from plasmas where shot={shot} and code_name='OMFITprofiles'", by_column=True)
-        for run in range(1, 9999):
-            runid = int('%06d%04d' % (shot, run))
-            if not len(runs) or runid not in runs['run_id']:
-                break
+        # add data to d3drdb (before MDS+ so that we can get a RUNID, only if it has not been allocated yet)
+        data = {
+            'shot': shot,
+            'experiment': 'DIII-D',
+            'run_type': 'user',
+            'tree': treename,
+            'start_time': np.min(self['time'].values),
+            'stop_time': np.max(self['time'].values),
+            'mdsserver': translate_MDSserver(server, ''),
+            'run_comment': comment,
+            #'runtag':tag
+        }
+        #'x_coord': x_coord,
+        command = "SpGetNextOmfitProfsID"
+        output = rdb.custom_procedure(command, commit=commit, **data)[-1]
+        runid = output.run_id
+        if runid == -1:
+            print("Error fetching available runid from SQL database")
+            return runid, treename
         print(f'Writing OMFITprofiles to MDS+ {runid}')
 
         # write to MDS+
@@ -426,35 +523,54 @@ class OMFITprofiles(OMFITncDataset):
             quantities=quantities,
             translator=lambda x: self.mds_translator(x),
         )
+        # Store meta data
+        # ====
         conn.write(treename=treename, shot=runid, node='__content__', data=';'.join(quantities))
+        conn.write(treename=treename, shot=runid, node='__x_coord__', data=x_coord)
+
+        coords_string = ''
+        disp_name_string = ''
+        for quant in quantities:
+            coords_string = ';'.join([coords_string, ','.join(self[quant].dims[::-1])])
+            # Upload reverses coord order, and above line accounts for it. But if upload behavior changes, this should
+            # also change.
+            try:
+                disp_name = self[quant].attrs['display_name']
+            except KeyError:
+                disp_name = ''
+            disp_name_string = ';'.join([disp_name_string, disp_name])
+
+        # Trim the initial ':' that result from the way this is built
+        if len(coords_string) > 1:
+            coords_string = coords_string[1:]
+        if len(disp_name_string) > 1:
+            disp_name_string = disp_name_string[1:]
+
+        conn.write(treename=treename, shot=runid, node='__coords__', data=coords_string)
+        conn.write(treename=treename, shot=runid, node='__dsp_name__', data=disp_name_string)
+        conn.write(treename=treename, shot=runid, node='__comment__', data=comment)
+
+        attrs_str = json.dumps(self.attrs, default=dumper)
+        conn.write(treename=treename, shot=runid, node='__attrs__', data=attrs_str)
 
-        # add data to d3drdb (after MDS+ so that this is done only if MDS+ write is successful)
-        data = {
-            'code_name': 'OMFITprofiles',
-            'shot': shot,
-            'experiment': 'DIII-D',
-            'run_type': 'user',
-            'tree': treename,
-            'start_time': np.min(self['time'].values),
-            'stop_time': np.max(self['time'].values),
-            'mdsserver': server,
-            'run_id': runid,
-        }
-        rdb.insert('plasmas', data, verbose=False)
         pprint(data)
 
         return runid, treename
 
-    def mds_translator(self, inv=None):
+    def mds_translator(self, inv=None, reverse=False):
         """
         Converts strings OMFITprofiles dataset keys to MDS+ nodes less than 12 chars long
 
-        :param inv: tring to which to apply the transformation
+        :param inv: string to which to apply the transformation
                     if `None` the transformation is applied to all of the OMFITprofiles.model_tree_quantities for sanity check
 
+        :param reverse: reverse the translation. Used to tranlate mds+ node names back to OMFITprofile names
+
         :return: transformed sting or if inv is None the `mapped_model_2_mds` and `mapped_mds_2_model` dictionaries
         """
         mapper = SortedDict()
+        mapper['_gradP_Vtor'] = 'gp_Vt'  # special case for 'Er_He_gradP_Vtor'
+        mapper['SAWTOOTH_'] = 'ST_'
         mapper['angular_momentum_density'] = 'mom_dens'
         mapper['angular_momentum'] = 'mom'
         mapper['midplane'] = 'mid'
@@ -469,11 +585,19 @@ class OMFITprofiles(OMFITncDataset):
         mapper['20Ne10'] = 'Ne'
         mapper['since_last'] = '_last'
         mapper['until_next'] = '_next'
+        mapper['_total'] = '_tot'
+        mapper['T_i_T_e'] = 'TiTe'
+        mapper['gradP'] = 'gp'
+        mapper['gamma'] = 'gm'
+        mapper['_ExB'] = 'eb'
 
+        if reverse:
+            mapper = SortedDict({y.lower(): x for x, y in mapper.items()})
+            inv = inv.lower()  # MDS+ is case insensitive, reverse mapping needs to adjust case for inputs
         if inv is not None:
             for match, sub in mapper.items():
                 inv = inv.replace(match, sub)
-            if len(inv) > 12:
+            if len(inv) > 12 and not reverse:
                 raise Exception(
                     f'MDS+ OMFITprofiles quantity is longer than 12 chars: {inv}\nUpdate the mds_translator function accordingly'
                 )
@@ -496,6 +620,60 @@ class OMFITprofiles(OMFITncDataset):
                 mapped_mds_2_model[item] = item0
             return mapped_model_2_mds, mapped_mds_2_model
 
+    def from_mds(self, server, runid):
+        from omfit_classes.omfit_mds import OMFITmds
+        import json
+
+        tree = OMFITmds(server=server, treename='OMFIT_profs', shot=runid)
+        contents = tree['__CONTENT__'].data()[0].split(";")
+        x_coord = tree['__x_coord__'].data()[0]
+        attrs_str = tree['__attrs__'].data()[0]
+        coords = tree['__coords__'].data()[0].split(";")
+        disp_names = tree['__dsp_name__'].data()[0].split(";")
+        comment = tree['__comment__'].data()[0]
+        if x_coord not in ['rho', 'psi_n']:
+            raise Exception(f"x_coord was recorded as {x_coord}. It is not a recognized radial coordinate.")
+
+        # Get the coords
+        n_coord = {}
+        for var in ['time', x_coord]:
+            if var not in contents:
+                # Tranlate exception to something that makes sense to user.
+                raise Exception(f"Coordinate {var} missing from MDS+ data!")
+            # Coord nodes by convention do not need translation, but might in the future
+            dat = tree[var].xarray()
+            dat = dat.rename(var)
+            dat = dat.rename({'dim_0': var})
+            self[var] = dat
+            n_coord[var] = len(dat.values)
+
+        for i, var in enumerate(contents):
+            if var in ['time', x_coord]:
+                # Skip, but process coord label and attrs
+                self[var].attrs['display_name'] = disp_names[i]
+            else:
+
+                node = self.mds_translator(inv=var)
+                dat = tree[node].xarray()
+                dat = dat.rename(var)
+                # Parse dims, and construct coord translator subset
+                ndim_data = len(dat.dims)
+                var_coords = coords[i].split(',')
+                ndim_coords = len(var_coords)
+                if ndim_data != ndim_coords:
+                    printw(f"Dimension count does not match record for {var}.")
+                else:
+                    rename_dict = {}
+                    for ii in np.arange(ndim_data):
+                        rename_dict[f'dim_{ii}'] = var_coords[ii]
+                    dat = dat.rename(rename_dict)
+                self[var] = dat
+
+        self.attrs = json.loads(attrs_str)
+        self.comment = comment
+        self.save()
+        return self
+
     def to_pFiles(self, eq, times=None, shot=None):
         """
         :param eq: ODS() or dict. (OMFITtree() is a dict)  Needs to contain equilibria information, either in the form
@@ -683,6 +861,20 @@ class OMFITprofiles(OMFITncDataset):
             return False
         return True
 
+    @dynaLoad
+    def reset_coords(self, names=None, drop=False):
+        """
+        Pass through implementation of Dataset.reset_coords(). Given names of coordinates, convert them to variables.
+        Unlike Dataset.reset_corrds(), however, this function modifies in place!
+
+        param names: Names of coords to reset. Cannot be index coords. Default to all non-index coords.
+
+        param drop: If True, drop coords instead of converting. Default False.
+        """
+        self._dataset = self._dataset.reset_coords(names=names, drop=drop)
+
+        return self
+
 
 class OMFITprofilesDynamic(OMFITncDynamicDataset):
     """
