@@ -1437,6 +1437,7 @@ class OMFITpythonTask(_OMFITpython):
         method='hybr',
         tol=None,
         options={'xtol': 1e-2, 'eps': 1e-2},
+        postfail='',
         reset_on_fail=True,
         **kw,
     ):
@@ -1472,6 +1473,9 @@ class OMFITpythonTask(_OMFITpython):
 
         :param options: keyword passed to scipy.optimize.root
 
+        :param postfail: string that is executed if execution of self throws an error
+                         *useful for gathering entries from the OMFIT tree
+
         :param reset_on_fail:  reset the tree if an exception or keyboard interrupt occurs
 
         :param \**kw: additional keywords passed to self.run()
@@ -1563,6 +1567,7 @@ class OMFITpythonTask(_OMFITpython):
                 return errors
 
             except (Exception, KeyboardInterrupt):
+                exec(postfail, globals(), locals())
                 if reset_on_fail:
                     # ensure tree returns to workable state if there's an exception
                     reset_tree(resets)
