@@ -21,7 +21,7 @@ try:
 except Exception as excp:
     warnings.warn('No RDB support: ' + repr(excp))
 
-__all__ = ['OMFITrdb', 'available_EFITs', 'translate_RDBserver', 'set_rdb_password', 'get_rdb_password']
+__all__ = ['OMFITrdb', 'available_efits_from_rdb', 'translate_RDBserver', 'set_rdb_password', 'get_rdb_password']
 
 _RDBserverDict = {
     # dictionary to translate mnemonic names to RDB servers stored in MainSettings['SERVER']
@@ -416,6 +416,33 @@ If this file is not available to you contact the GA computer group: iris-support
 
         return res
 
+    def custom_procedure(self, procedure, commit=True, **arguments):
+        """
+        Pass an arbitrary custom procedure to the sql database
+
+        :param procedure: string
+            A string that represents the custom procedure to be called
+        :param commit (bool): If set to False it will not commit the data to the coderunrdb. This should be done when running a
+            jenkins test, otherwise it may attempt to write data to the same shot/runid twice and throw an error.
+        :return: Dict
+            Output list of pyodbc rows returned by the custom query
+        """
+        arguments = ', '.join(
+            [
+                "@%s='%s'" % (arg, arguments[arg]) if isinstance(arguments[arg], str) else "@%s=%s" % (arg, arguments[arg])
+                for arg in arguments
+            ]
+        )
+        command = "exec %s " % (procedure) + arguments
+        if self.cnxn is None or self.cursor is None:
+            self._connect()
+            self.cursor.execute("use " + self.db)
+        self.cursor.execute(command)
+        rows = self.cursor.fetchall()
+        if commit:
+            self.cnxn.commit()
+        return rows
+
     def alter_add(self, table, column_name, column_type, commit=True, verbose=True):
         """
         Alter table in SQL database by adding a column
@@ -837,11 +864,11 @@ If this file is not available to you contact the GA computer group: iris-support
             connection['PORT'] = rdb_server[3]
             if 'instance' in MS_server0:
                 del connection['PORT']
-                connection['SERVER'] = f"{connection['SERVER']}\{MS_server0['instance']}"
+                connection['SERVER'] = rf"{connection['SERVER']}\{MS_server0['instance']}"
             driver = 'tdsodbc'
             if 'driver' in MS_server0:
-                connection['DRIVER'] = driver = MS_server0['driver']
-            if isinstance(driver, str) and driver and os.sep not in driver:
+                connection['DRIVER'] = driver = evalExpr(MS_server0['driver'])
+            if isinstance(driver, str) and driver and os.sep not in driver and driver not in pyodbc.drivers():
                 connection['DRIVER'] = find_library(driver)
             if not isinstance(connection['DRIVER'], str) or not connection['DRIVER']:
                 raise OMFITexception(
@@ -866,7 +893,7 @@ If this file is not available to you contact the GA computer group: iris-support
         return connection
 
 
-def available_EFITs(scratch, device, shot, default_snap_list=None, format='{tree}'):
+def available_efits_from_rdb(scratch, device, shot, default_snap_list=None, format='{tree}', **kw):
     """
     :param scratch: dict
         dictionary where the information is cached (to avoid querying SQL database multiple times)
@@ -883,22 +910,27 @@ def available_EFITs(scratch, device, shot, default_snap_list=None, format='{tree
     :param format: string
         format in which to write list of available efits (tree, by, com, drun, runid) are possible options
 
-    :return: dict
+    :param **kw: quietly accepts and ignores other keywords for compatibility with other similar functions
+
+    :return: (dict, str)
         dictionary with list of available options formatted as {text:format}
+        information about the discovered EFITs
     """
-    efit_sql = 'EFIT_SQL_%s' % str(shot)
+    efit_sql = f'EFIT_SQL_{device}_{shot}'
     if default_snap_list is None:
         default_snap_list = {'': ''}
     snap_list = copy.deepcopy(default_snap_list)
     help = ''
-    if is_device(device, 'DIII-D'):
+    scratch.setdefault('searched_in', [])
+    scratch['searched_in'] += ['rdb']
+    if is_device(device, ['DIII-D', 'NSTX', 'NSTXU']):
         try:
             query = "SELECT * FROM plasmas WHERE shot=" + str(shot) + " AND code_name='EFIT' AND deleted!=1 ORDER BY tree"
             try:
                 if scratch[efit_sql].query != query:
-                    scratch[efit_sql] = OMFITrdb(query, 'code_rundb')
+                    scratch[efit_sql] = OMFITrdb(query, 'code_rundb', server=device)
             except Exception:
-                scratch[efit_sql] = OMFITrdb(query, 'code_rundb')
+                scratch[efit_sql] = OMFITrdb(query, 'code_rundb', server=device)
             runids = scratch[efit_sql].across("['*']['shot']")
             try:
                 runids = scratch[efit_sql].across("['*']['run_id']")
@@ -930,11 +962,15 @@ def available_EFITs(scratch, device, shot, default_snap_list=None, format='{tree
 
                 else:
                     snap_list['[' + item + '] ' + ' (???)'] = format.format(tree=tree, runid=runid, by=by, com=com, drun=drun, tag=tag)
+                scratch['available_efits_from_rdb_success'] = True
 
         except Exception as _excp:
+            scratch['available_efits_from_rdb_success'] = False
             raise
             printe('Could not retrieve list of available EFIT runs: ' + repr(_excp))
             snap_list = {'EFIT01': 'EFIT01', 'EFIT02': 'EFIT02'}
             help = 'Enter EFIT tree by hand\n\nCould not retrieve list of available EFIT runs:\n' + repr(_excp)
+    else:
+        scratch['available_efits_from_rdb_success'] = False
     help = help.strip()
     return snap_list, help
