@@ -140,9 +140,9 @@ class OMFITtranspNamelist(OMFITnamelist):
         return tmp
 
     def _checkSetitem(self, key, val):
-        '''
+        """
         Allow non-string keys
-        '''
+        """
         return key, val
 
 
@@ -528,11 +528,11 @@ class OMFITtranspMultigraph(SortedDict):
                         fig = t.fig
                         v = fig.view
                         # re-produce fills
-                        l, = axx.plot(v._hslice_xdata, v._hslice_mean, label=k, **v.plot_options)
+                        (l,) = axx.plot(v._hslice_xdata, v._hslice_mean, label=k, **v.plot_options)
                         axx.fill_between(
                             v._hslice_xdata, v._hslice_mean - v._hslice_std, v._hslice_mean + v._hslice_std, color=l.get_color(), alpha=0.3
                         )
-                        l, = axy.plot(v._vslice_xdata, v._vslice_mean, label=k, **v.plot_options)
+                        (l,) = axy.plot(v._vslice_xdata, v._vslice_mean, label=k, **v.plot_options)
                         axy.fill_between(
                             v._vslice_xdata, v._vslice_mean - v._vslice_std, v._vslice_mean + v._vslice_std, color=l.get_color(), alpha=0.3
                         )
@@ -575,7 +575,7 @@ class OMFITtranspMultigraph(SortedDict):
 
 
 def check_TRANSP_run(runid, project=None, tunnel=None):
-    '''
+    """
     Function that checks the status of a TRANSP run as reported
     by the TRANSP MONITOR GRID website: https://w3.pppl.gov/transp/transpgrid_monitor
 
@@ -588,7 +588,7 @@ def check_TRANSP_run(runid, project=None, tunnel=None):
     :return: * None if no matching runid/project is found
              * tuple with 1) True/None/False if run went ok, run is waiting, run failed
                      and  2) dictionary with parsed HTML information
-    '''
+    """
 
     socks = {}
     if tunnel is not None:
@@ -636,7 +636,7 @@ def check_TRANSP_run(runid, project=None, tunnel=None):
 
 
 def wait_TRANSP_run(runid, project=None, t_check=5, verbose=True, tunnel=None):
-    '''
+    """
     Function that waits for a TRANSP run to end as reported
     by the TRANSP MONITOR GRID website: https://w3.pppl.gov/transp/transpgrid_monitor
 
@@ -653,7 +653,7 @@ def wait_TRANSP_run(runid, project=None, t_check=5, verbose=True, tunnel=None):
     :return: * None if no matching runid/project is found
              * tuple with 1) True/False if run went ok or run failed
                      and  2) dictionary with parsed HTML information
-    '''
+    """
 
     aborted = tk.BooleanVar()
     aborted.set(False)
@@ -726,7 +726,7 @@ def wait_TRANSP_run(runid, project=None, t_check=5, verbose=True, tunnel=None):
 
 
 def next_available_TRANSP_runid(runid, project, increment=1, skiplist=[], server=None):
-    '''
+    """
     Function which checks what MDS+ tree entries are available
 
     :param runid: runid to start checking from (included)
@@ -740,7 +740,7 @@ def next_available_TRANSP_runid(runid, project, increment=1, skiplist=[], server
     :param server: MDS+ TRANSP server to be queried [e.g. 'atlas.gat.com', or 'transpgrid.pppl.gov']
 
     :return: tuple with next available runid (as integer with format shotXXYY) and augmented skiplist
-    '''
+    """
 
     # handle letter in runid
     try:
@@ -823,9 +823,9 @@ class OMFITplasmastate(OMFITnc):
     }
 
     def calcQ(self):
-        '''
+        """
         :return: fusion gain
-        '''
+        """
         auxi = 0
         for k in ['pbi', 'picth', 'pmini']:
             auxi += np.sum(self[k]['data'])
@@ -844,9 +844,9 @@ class OMFITplasmastate(OMFITnc):
         return (fus_neut + fus_alpha) / aux
 
     def summary_sources(self):
-        '''
+        """
         Print summary of integrated sources
-        '''
+        """
         for item in sorted(list(self.sources.keys())):
             if item.startswith('p'):
                 print('%s:% 4d MW      [%s]' % (item.ljust(10), np.sum(self[item]['data']) / 1e6, self.sources[item]))
@@ -856,7 +856,7 @@ class OMFITplasmastate(OMFITnc):
                 print('%s:% 4d N*m^2   [%s]' % (item.ljust(10), int(np.sum(self[item]['data'])), self.sources[item]))
 
     def to_omas(self, ods=None, time_index=0, update=['core_profiles', 'equilibrium', 'core_sources']):
-        '''
+        """
         translate TRANSP plasmastate file (output of TRXPL and plasmastate of SWIM) to OMAS data structure
 
         :param ods: input ods to which data is added
@@ -866,7 +866,7 @@ class OMFITplasmastate(OMFITnc):
         :update: list of IDS to update from statefile
 
         :return: ODS
-        '''
+        """
         from omas import ODS, omas_environment
 
         cocosio = 2  # need to confirm that indeed TRANSP plasmastate is in COCOS 2
@@ -1073,9 +1073,9 @@ class OMFITtranspOutput(OMFITdataset):
     """
 
     def __init__(self, transp_out):
-        '''
+        """
         :param transp_out: OMFITnc file, OMFITmds TRANSP tree, or string path to NetCDF file
-        '''
+        """
         if isinstance(transp_out, str):
             transp_out = OMFITnc(transp_out)
         if isinstance(transp_out, OMFITmds):
@@ -1180,7 +1180,7 @@ class OMFITfbm(OMFITnc):
     """
 
     def plot(self, rf=None, zf=None, cartesian=True, fig=None):
-        '''
+        """
         Plot distribution function
 
         :param rf: radial location where to show data in velocity space
@@ -1188,7 +1188,7 @@ class OMFITfbm(OMFITnc):
         :param zf: vertical location where to show data in velocity space
 
         :return: figure handle
-        '''
+        """
         f = self['F_D_NBI']['data']
         energy = self['E_D_NBI']['data']
         pitch = self['A_D_NBI']['data']
@@ -1345,7 +1345,7 @@ class OMFITfbm(OMFITnc):
         return ax
 
     def to_omas(self, ods=None, time_index=0):
-        '''
+        """
         Save NUBEAM distribution function to OMAS
 
         :param ods: input ods to which data is added
@@ -1353,7 +1353,7 @@ class OMFITfbm(OMFITnc):
         :param time_index: time index to which data is added
 
         :return: updated ODS
-        '''
+        """
         if ods is None:
             ods = ODS()
 
@@ -1771,7 +1771,7 @@ def aol(d, rmnmp=None):
     if isinstance(d, OMFITtranspMultigraph):
         dnew = copy.copy(d)
         for k, v in list(dnew['CONTENT'].items()):
-            dnew['CONTENT'][k] = vint(v, rmnmp)
+            dnew['CONTENT'][k] = aol(v, rmnmp)
         return dnew
 
     if rmnmp == None:
@@ -1801,6 +1801,68 @@ OMFITtranspData.aol = aol
 OMFITtranspMultigraph.aol = aol
 
 
+def tavg(d, time=None, avgtime=None):
+    """
+    Time average data
+
+    :param d: OMFITtranspData object from the TRANSP tree.
+    :param time: Center time in seconds
+    :param avgtime: Averaging window (+/-) in seconds
+
+    :return: For 1D input uncertainty uarray of the data time avarge and standard deviation.
+             For 2D input uncertainty uarray of the profile with time avarge and standard deviation.
+
+    **Example:**
+    Assuming data in root['OUTPUTS']['TRANSP_OUTPUT']
+
+    time = 2.0
+    avgtime = 0.1
+    # 1D
+    tr_neutt = OMFITtranspData(root['OUTPUTS']['TRANSP_OUTPUT'], 'NEUTT')
+    tr_neutt_tavg = tr_neutt.tavg(time=time, avgtime=avgtime)
+    tr_neutt.plot()
+    ax = gca()
+    uband([time-avgtime, time+avgtime],np.repeat(tr_neutt_tavg,2),ax=ax)
+    ax.text(time,nominal_values(tr_neutt_tavg)+std_devs(tr_neutt_tavg),
+            '{0:0.3g}+/-{1:0.3g}'.format(nominal_values(tr_neutt_tavg),std_devs(tr_neutt_tavg)))
+
+    # 2D
+    tr_ne = OMFITtranspData(root['OUTPUTS']['TRANSP_OUTPUT'],'NE')
+    tr_ne_tavg = tr_ne.tavg(time=time, avgtime=avgtime)
+    figure()
+    tr_ne.plot(slice_axis=1,slice_at=time)
+    ax = gca()
+    uband(tr_ne['DIM_OF'][0][0,:],tr_ne_tavg,ax=ax)
+
+    """
+
+    if time is None or avgtime is None:
+        raise (OMFITexception("Must set time and average time"))
+
+    # Determine if 1D or 2D data
+    if len(d['DIM_OF']) == 1 and d['UNITS_DIM_OF'][0] == 's':
+        # 1D
+        idx = np.where((d['DIM_OF'][0] >= time - avgtime) * (d['DIM_OF'][0] <= time + avgtime))[0]
+        if len(idx):
+            ud = uarray(np.mean(d['DATA'][idx]), np.std(d['DATA'][idx]))
+        else:
+            raise (OMFITexception("Averaging window empty."))
+    elif len(d['DIM_OF']) == 2 and d['UNITS_DIM_OF'][1] == 's':
+        # 2D
+        idx = np.where((d['DIM_OF'][1][:, 0] >= time - avgtime) * (d['DIM_OF'][1][:, 0] <= time + avgtime))[0]
+        if len(idx):
+            ud = uarray(np.mean(d['DATA'][idx, :], axis=0), np.std(d['DATA'][idx, :], axis=0))
+        else:
+            raise (OMFITexception("Averaging window empty."))
+    else:
+        raise (OMFITexception("Unrecognized dimensions of quantity to average."))
+
+    return ud
+
+
+OMFITtranspData.tavg = tavg
+
+
 ############################################
 if '__main__' == __name__:
     test_classes_main_header()
