@@ -14,14 +14,10 @@ except ImportError as _excp:
 
 from omfit_classes.omfit_ascii import OMFITascii
 from omfit_classes.sortedDict import SortedDict
+from omfit_classes.utils_plot import View2d, View3d
 
 import numpy as np
 
-try:
-    from omfit_plot import View2d, View3d
-except ImportError as _excp:
-    warnings.warn('Unable to import omfit_plot; plotting will not work: %s' % repr(_excp))
-
 __all__ = ['OMFITuFile']
 
 
