@@ -14,6 +14,7 @@ import re
 import pprint
 import warnings
 import traceback
+from collections.abc import MutableMapping
 
 __all__ = [
     'hide_ptrn',
@@ -300,7 +301,15 @@ def traverseLocation(inv):
 
 @_available_to_user_util
 def traverse(
-    self, string='', level=100, split=True, onlyDict=False, onlyLeaf=False, skipDynaLoad=False, noSubmodules=False, traverse_classes=(dict,)
+    self,
+    string='',
+    level=100,
+    split=True,
+    onlyDict=False,
+    onlyLeaf=False,
+    skipDynaLoad=False,
+    noSubmodules=False,
+    traverse_classes=(MutableMapping,),
 ):
     """
     Returns a string or list of strings describing the path of every entry/subentries in the dictionary
@@ -325,7 +334,7 @@ def traverse(
     """
     string_in = string
     string_out = ''
-    if isinstance(self, dict):
+    if isinstance(self, (MutableMapping,)):
         keys = list(self.keys())
     elif isinstance(self, (list, tuple)):
         keys = list(range(len(self)))
